#!/usr/bin/env python
import sys, os, shutil, math, commands, glob
from pylab import *
from PyFoam.RunDictionary.BlockMesh import BlockMesh 
from subprocess import Popen
from subprocess import call

# divSchems can be found in the following directories:
#    $WM_PROJECT_DIR/src/finiteVolume/interpolation/surfaceInterpolation/schemes
#    $WM_PROJECT_DIR/src/finiteVolume/interpolation/surfaceInterpolation/limitedSchemes
divSchemes=[ 'upwind', 'linear', 'limitedLinearV']
#divSchemes=[ 'linear', 'cubic', 'downwind', 'midPoint', 'MUSCL', 'Minmod', 'QUICK', 
#		'filteredLinear', 'upwind', 'vanLeer']

ddtSchemes=['Euler','backward','CrankNicholson 1.0']  # currently not used

# set original directory to top of tree
origdir=os.getcwd()

# Run everything from results directory & create figures directory for holding all plots
if os.path.isdir('figures'):    	# does figures directory exist?
	shutil.rmtree('figures')	# yes, remove old directory
os.mkdir('figures')                     # create figures dir
if os.path.isdir('results'):            # does results directory exist?
	shutil.rmtree('results')	# yes, remove old directory
os.mkdir('results')                     # create results dir
os.chdir('results')

# Loop over available divSchemes

for divScheme in divSchemes:

	if os.path.isdir(divScheme):    	# does directory named dir exist?
		shutil.rmtree(divScheme)	# yes, remove old directory
	os.mkdir(divScheme)
	os.chdir(divScheme)

        # Loop over grid refinement:  baseline is coarse mesh of 8 x 8
	#    refine mesh using pyFoam function BlockMesh.refineMesh and scalar refinement of 2

	meshDir = ['mesh8','mesh16','mesh32','mesh64']

	maxError = []
	aveError = []
	gridSize= [8.0, 4.0, 2.0, 1.0]

	for mesh in meshDir:

		print 'Executing',divScheme,mesh 
		meshIndex =meshDir.index(mesh) 

		if os.path.isdir(mesh):		# does directory named mesh exist?
			shutil.rmtree(mesh)      # yes, remove old directory

		# use pyFoam to clone the baseline case
		cmd='pyFoamCloneCase.py ../../baseline %s' % mesh 
		pipefile = open('output', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile,stderr=pipefile)
		pipefile.close()
		os.remove('output')

		# cd into case directory
		os.chdir(mesh)					

		# change the divScheme for "div(phi,U)  Gauss linear;" in the fvSchemes file
		infilename='system/fvSchemes'
		outfilename='system/fvSchemesTemp'
		ifile = open( infilename, 'r')
		ofile = open(outfilename, 'w')
		lines = ifile.readlines()
		for line in lines:
			words = line.split()
			for word in words:
				index =words.index(word) 
				if word == 'div(phi,U)': 
					if divScheme == 'limitedLinearV':
						words[index+2] = divScheme+' 1;'
					else:
						words[index+2] = divScheme+';'
					continue
			newline=' '.join(words)
			ofile.write('%s\n' % newline)
		ifile.close()
		ofile.close()
		os.remove(infilename)
		os.rename(outfilename,infilename)

		# set block size in blockMeshDict
		dict="constant/polyMesh/blockMeshDict"  	# blockMeshDict
		bm=BlockMesh(dict)                              # define bm to be a pyFoam BlockMesh object 
		number=meshDir.index(mesh)                      # find index of current mesh: 0, 1, 2, 3
		bm.refineMesh((2**number,2**number,1))         	# refineMesh by factor of 2^number over baseline
		meshDim = 8*2**number

		# run blockMesh
		cmd='blockMesh'
		pipefile = open('output', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile)
		pipefile.close()
		os.remove('output')

		# run funkySetFields to set initial conditions 
		cmd='funkySetFields -time 0'
		pipefile = open('output', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile)
		pipefile.close()
		os.remove('output')

		# run icoFoam to solve Navier Stokes equations
		cmd='icoFoam > log 2>&1'
		pipefile = open('output', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile)
		pipefile.close()
		os.remove('output')

		# run custom post-processor for computing analytical solution at each time step, 
		# and for computing comparison error
		cmd='analyticalSolutionTaylorVortex -time 1'
		pipefile = open('ASTVOutput', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile)
		pipefile.close()

		# extract maximum error at endTime (time = 1) from STDOUT
		pipefile = open('ASTVOutput', 'r')
		lines = pipefile.readlines()
		for line in lines:
			words = line.split()
			for word in words:
				if word == 'Maximum': 
					value=float(words[len(words)-1])
					maxError.append(value)
				if word == 'Average': 
					value=float(words[len(words)-1])
					aveError.append(value)
		pipefile.close()

		# run sample utility
		cmd='sample -time 1'
		pipefile = open('output', 'w')
		retcode = call(cmd,shell=True,stdout=pipefile)
		pipefile.close()
		os.remove('output')

		# construct contour & vector plots
		cmd='python '+origdir+'/tools/plotContours.py '+ str(meshDim)+' '+divScheme+' '+mesh
		retcode = call(cmd,shell=True)
		cmd='python '+origdir+'/tools/plotVectors.py '+ str(meshDim)+' '+divScheme+' '+mesh
		retcode = call(cmd,shell=True)

                # move.eps image files to figures directory
		filelist=glob.glob('*.eps')
		for file in filelist:
			shutil.move(file, origdir+"/figures/.")

		os.chdir("..")

	# construct line plots comparing all meshes for a given divScheme
	cmd='python '+origdir+'/tools/plotLines.py '+divScheme
	retcode = call(cmd,shell=True)
	filelist=glob.glob('*.eps')
	for file in filelist:
		shutil.move(file, origdir+"/figures/.")

	# construct data for error plots vs. grid resolution (maxError and aveError)
	outfilename='spatialError'
	ofile = open(outfilename, 'w')
	mE = array(maxError)
	aE = array(aveError)
	for i in range(len(maxError)):
		out=[str(gridSize[i]),str(mE[i]),str(aE[i])]
		newline=' '.join(out)
		ofile.write('%s\n' % str(newline))
        ofile.close()

	os.chdir("..")

cmd='python '+origdir+'/tools/plotError.py'
retcode = call(cmd,shell=True)
filelist=glob.glob('*.eps')
for file in filelist:
	shutil.move(file, origdir+"/figures/.")
